searchData={"items":[{"type":"module","title":"ExEssentials","doc":"","ref":"ExEssentials.html"},{"type":"behaviour","title":"ExEssentials.Web.Plugs.DisableServices","doc":"A `Plug` that conditionally disables specific controller actions based on a feature flag,\nwith optional support for group-based evaluation using `FunWithFlags`.","ref":"ExEssentials.Web.Plugs.DisableServices.html"},{"type":"behaviour","title":"Overview - ExEssentials.Web.Plugs.DisableServices","doc":"This plug allows developers to temporarily disable specific Phoenix actions\n(e.g., `:create`, `:delete`) by checking if a feature flag is enabled for the current context (typically, the current user or realm).\n\nIt works by intercepting the connection and returning a `503 Service Unavailable`\nresponse if the feature flag is enabled and the requested action is among the disabled ones.\n\nGroup-based targeting is supported through `FunWithFlags`, using a struct that includes\nthe current user as `current_user`.","ref":"ExEssentials.Web.Plugs.DisableServices.html#module-overview"},{"type":"behaviour","title":"Configuration - ExEssentials.Web.Plugs.DisableServices","doc":"You must provide the following options when using the plug:\n\n  * `:disabled_actions` – a list of controller actions (atoms) that should be disabled if the flag is active.\n  * `:flag_name` – the name of the feature flag to check (defaults to `:disable_services_enabled`).\n\nThis plug uses `FunWithFlags.enabled?/2` with a custom context struct that supports group checks via `FunWithFlags.Group`.","ref":"ExEssentials.Web.Plugs.DisableServices.html#module-configuration"},{"type":"behaviour","title":"Dependencies - ExEssentials.Web.Plugs.DisableServices","doc":"To use this plug, your application must be configured with:\n\n```elixir\n{:fun_with_flags, \"~> 1.13\"},\n{:ecto_sql, \"~> 3.4\"},\n{:postgrex, \">= 0.0.0\"},\n```\n\nAnd you must configure the `FunWithFlags` adapter to use Ecto with a repo.","ref":"ExEssentials.Web.Plugs.DisableServices.html#module-dependencies"},{"type":"behaviour","title":"Usage - ExEssentials.Web.Plugs.DisableServices","doc":"","ref":"ExEssentials.Web.Plugs.DisableServices.html#module-usage"},{"type":"behaviour","title":"Simple usage (default behavior) - ExEssentials.Web.Plugs.DisableServices","doc":"By default, the plug uses `conn.assigns.user_name` or `conn.params[\"user_name\"]` to resolve the group.\n\n```elixir\nplug ExEssentials.Web.Plugs.DisableServices,\n  disabled_actions: [:create, :delete],\n  flag_name: :services_disabled\n```","ref":"ExEssentials.Web.Plugs.DisableServices.html#module-simple-usage-default-behavior"},{"type":"behaviour","title":"Customized usage with use and @impl - ExEssentials.Web.Plugs.DisableServices","doc":"To change how the group name is extracted from the connection (e.g., use `realm` instead of `user_name`),\nyou can define your own module and override the `get_current_user/1` callback:\n\n```elixir\ndefmodule MyApp.Web.Plugs.DisableServices do\n  use ExEssentials.Web.Plugs.DisableServices\n\n  @impl ExEssentials.Web.Plugs.DisableServices\n  def get_current_user(%Plug.Conn{assigns: %{realm: realm}}), do: realm\n  def get_current_user(%Plug.Conn{params: params}), do: Map.get(params, \"realm\")\n  def get_current_user(_), do: nil\nend\n```\n\nAnd in your router or endpoint:\n\n```elixir\nplug MyApp.Web.Plugs.DisableServices,\n  disabled_actions: [:create, :delete],\n  flag_name: :services_disabled\n```","ref":"ExEssentials.Web.Plugs.DisableServices.html#module-customized-usage-with-use-and-impl"},{"type":"behaviour","title":"Example with FunWithFlags - ExEssentials.Web.Plugs.DisableServices","doc":"To enable or disable the flag at runtime:\n\n```elixir\nFunWithFlags.enable(:services_disabled)\nFunWithFlags.disable(:services_disabled, for_group: \"admin\")\n```\n\nThis will enable the flag globally, but disable it for users in the \"admin\" group.","ref":"ExEssentials.Web.Plugs.DisableServices.html#module-example-with-funwithflags"},{"type":"behaviour","title":"Notes - ExEssentials.Web.Plugs.DisableServices","doc":"* If `FunWithFlags` is not available at runtime (e.g., not installed), the plug assumes the flag is **enabled by default** and proceeds with normal request handling.\n* The `ExEssentials.Web.Plugs.DisableServices` struct implements the `FunWithFlags.Group` protocol to support group-based targeting.","ref":"ExEssentials.Web.Plugs.DisableServices.html#module-notes"},{"type":"function","title":"ExEssentials.Web.Plugs.DisableServices.call/2","doc":"Callback implementation for  Plug.call/2 .","ref":"ExEssentials.Web.Plugs.DisableServices.html#call/2"},{"type":"function","title":"ExEssentials.Web.Plugs.DisableServices.get_current_user/1","doc":"Extracts the current user identifier from the connection for use in group-based flag evaluation.\n\nThis function looks first in `conn.assigns[:user_name]`, and if not found, in `conn.params[\"user_name\"]`.","ref":"ExEssentials.Web.Plugs.DisableServices.html#get_current_user/1"},{"type":"function","title":"Examples - ExEssentials.Web.Plugs.DisableServices.get_current_user/1","doc":"iex> conn = %Plug.Conn{assigns: %{user_name: \"admin\"}}\n    iex> ExEssentials.Web.Plugs.DisableServices.get_current_user(conn)\n    \"admin\"\n\n    iex> conn = %Plug.Conn{params: %{\"user_name\" => \"admin\"}}\n    iex> ExEssentials.Web.Plugs.DisableServices.get_current_user(conn)\n    \"admin\"\n\n    iex> ExEssentials.Web.Plugs.DisableServices.get_current_user(%Plug.Conn{})\n    nil","ref":"ExEssentials.Web.Plugs.DisableServices.html#get_current_user/1-examples"},{"type":"callback","title":"ExEssentials.Web.Plugs.DisableServices.get_current_user/1","doc":"","ref":"ExEssentials.Web.Plugs.DisableServices.html#c:get_current_user/1"},{"type":"function","title":"ExEssentials.Web.Plugs.DisableServices.init/1","doc":"Callback implementation for  Plug.init/1 .","ref":"ExEssentials.Web.Plugs.DisableServices.html#init/1"},{"type":"module","title":"ExEssentials.Web.Plugs.RequestValidator","doc":"A `Plug` for validating request parameters using `Ecto.Changeset`.\n\nThis plug enables modular and scalable validation of incoming request parameters by delegating the validation logic to a specific module (the `:validator`). It dynamically calls a function named after the current `:phoenix_action`, passing the request `params` as an argument. The function must return an `%Ecto.Changeset{}`.\n\nIf the changeset is valid, the request proceeds as usual. If invalid, a `400 Bad Request` response is sent in the `application/problem+json` format.","ref":"ExEssentials.Web.Plugs.RequestValidator.html"},{"type":"module","title":"Options - ExEssentials.Web.Plugs.RequestValidator","doc":"* `:validator` – **(required)** The module responsible for validation. It must implement a function for each action, with the same name as the `:phoenix_action` and an arity of 1 (accepting the `params`).","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-options"},{"type":"module","title":"Example Usage - ExEssentials.Web.Plugs.RequestValidator","doc":"Given a controller with the `:create` action and a validator module:\n\n    defmodule MyAppWeb.Validators.Types.User.Preferences do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      @themes ~w(light dark)\n      @languages ~w(en pt-br)\n      @fields ~w(theme language)a\n\n      @primary_key false\n      embedded_schema do\n        field(:theme, :string)\n        field(:language, :string)\n      end\n\n      def changeset(schema, params) do\n        schema\n        |> cast(params, @fields)\n        |> validate_required(@fields)\n        |> validate_inclusion(:theme, @themes)\n        |> validate_inclusion(:language, @languages)\n      end\n    end\n\n    defmodule MyAppWeb.Validators.Types.User.Course do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      @fields ~w(name duration level)a\n      @levels ~w(beginner intermediary advanced)\n\n      @primary_key false\n      embedded_schema do\n        field(:name, :string)\n        field(:duration, :integer)\n        field(:level, :string)\n      end\n\n      def changeset(schema, params) do\n        schema\n        |> cast(params, @fields)\n        |> validate_required(@fields)\n        |> validate_number(:duration, greater_than: 0)\n        |> validate_inclusion(:level, @levels)\n      end\n    end\n\n    defmodule MyAppWeb.Validators.User do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      alias MyAppWeb.Validators.Types.User.Course\n      alias MyAppWeb.Validators.Types.User.Preferences\n      alias MyAppWeb.Validators.User\n\n      @fields ~w(name email)a\n\n      schema \"users\" do\n        field(:name, :string)\n        field(:email, :string)\n\n        embeds_one(:preferences, Preferences)\n        embeds_many(:courses, Course)\n      end\n\n      def create(params) do\n        %User{}\n        |> cast(params, @fields)\n        |> validate_required(@fields)\n        |> cast_embed(:preferences, required: true)\n        |> cast_embed(:courses, required: true)\n      end\n    end\n\nYou can plug the validator into your controller or router pipeline:\n\n    plug ExEssentials.Web.Plugs.RequestValidator, validator: MyAppWeb.Validators.User","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-example-usage"},{"type":"module","title":"Error Response Format - ExEssentials.Web.Plugs.RequestValidator","doc":"When validation fails, the response follows the structure inspired by\n[RFC 7807 - Problem Details for HTTP APIs](https://datatracker.ietf.org/doc/html/rfc7807):\n\n    {\n      \"errors\": [\n        {\n          \"code\": \"invalid_parameter\",\n          \"detail\": \"The 'name' field can't be blank\",\n          \"title\": \"Invalid request parameters\"\n        },\n        {\n          \"code\": \"invalid_parameter\",\n          \"detail\": \"The 'courses.[1].level' field is invalid\",\n          \"title\": \"Invalid request parameters\"\n        },\n        {\n          \"code\": \"invalid_parameter\",\n          \"detail\": \"The 'preferences.language' field can't be blank\",\n          \"title\": \"Invalid request parameters\"\n        },\n        {\n          \"code\": \"invalid_parameter\",\n          \"detail\": \"The 'preferences.theme' field is invalid\",\n          \"title\": \"Invalid request parameters\"\n        }\n      ],\n      \"status_code\": 400\n    }\n\n\nEach validation error includes:\n  * `code` – configurable error code.\n  * `detail` – human-readable description of the error.\n  * `title` – general error title.","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-error-response-format"},{"type":"module","title":"Configuration - ExEssentials.Web.Plugs.RequestValidator","doc":"You can customize certain aspects via your `config.exs`:\n\n    config :ex_essentials, :web_request_validator,\n      json_library: Jason,\n      error_code: :invalid_parameter,\n      error_title: \"Invalid request parameters\"\n\nBy default, the module uses Jason to encode JSON responses, assigning the\nerror code :invalid_parameter and the error title \"Invalid request parameters\".","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-configuration"},{"type":"module","title":"Notes - ExEssentials.Web.Plugs.RequestValidator","doc":"* This plug assumes Phoenix is being used and requires `:phoenix_action` to be set in `conn.private`.\n  * `Ecto` is required for working with changesets.","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-notes"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}